@@ -6,26 +6,27 @@ import queue
 import threading
 import time
 from typing import Any, Callable, Dict, List, Iterable, Optional, Set, Tuple,\
-    TYPE_CHECKING, Union
+    Type, TYPE_CHECKING, Union
 
 from ray.util.debug import log_once
 from ray.rllib.evaluation.collectors.sample_collector import \
-    _SampleCollector
+    SampleCollector
 from ray.rllib.evaluation.collectors.simple_list_collector import \
-    _SimpleListCollector
+    SimpleListCollector
 from ray.rllib.evaluation.episode import MultiAgentEpisode
 from ray.rllib.evaluation.rollout_metrics import RolloutMetrics
 from ray.rllib.evaluation.sample_batch_builder import \
     MultiAgentSampleBatchBuilder
+from ray.rllib.env.base_env import BaseEnv, ASYNC_RESET_RETURN
+from ray.rllib.env.wrappers.atari_wrappers import get_wrapper_by_cls, \
+    MonitorEnv
+from ray.rllib.models.preprocessors import Preprocessor
+from ray.rllib.offline import InputReader
 from ray.rllib.policy.policy import clip_action, Policy
 from ray.rllib.policy.tf_policy import TFPolicy
-from ray.rllib.models.preprocessors import Preprocessor
-from ray.rllib.utils.filter import Filter
-from ray.rllib.env.base_env import BaseEnv, ASYNC_RESET_RETURN
-from ray.rllib.env.atari_wrappers import get_wrapper_by_cls, MonitorEnv
-from ray.rllib.offline import InputReader
 from ray.rllib.utils.annotations import override, DeveloperAPI
 from ray.rllib.utils.debug import summarize
+from ray.rllib.utils.filter import Filter
 from ray.rllib.utils.numpy import convert_to_numpy
 from ray.rllib.utils.spaces.space_utils import flatten_to_single_ndarray, \
     unbatch
@@ -119,25 +120,28 @@ class SyncSampler(SamplerInput):
     """Sync SamplerInput that collects experiences when `get_data()` is called.
     """
 
-    def __init__(self,
-                 *,
-                 worker: "RolloutWorker",
-                 env: BaseEnv,
-                 policies: Dict[PolicyID, Policy],
-                 policy_mapping_fn: Callable[[AgentID], PolicyID],
-                 preprocessors: Dict[PolicyID, Preprocessor],
-                 obs_filters: Dict[PolicyID, Filter],
-                 clip_rewards: bool,
-                 rollout_fragment_length: int,
-                 callbacks: "DefaultCallbacks",
-                 horizon: int = None,
-                 multiple_episodes_in_batch: bool = False,
-                 tf_sess=None,
-                 clip_actions: bool = True,
-                 soft_horizon: bool = False,
-                 no_done_at_end: bool = False,
-                 observation_fn: "ObservationFunction" = None,
-                 _use_trajectory_view_api: bool = False):
+    def __init__(
+            self,
+            *,
+            worker: "RolloutWorker",
+            env: BaseEnv,
+            policies: Dict[PolicyID, Policy],
+            policy_mapping_fn: Callable[[AgentID], PolicyID],
+            preprocessors: Dict[PolicyID, Preprocessor],
+            obs_filters: Dict[PolicyID, Filter],
+            clip_rewards: bool,
+            rollout_fragment_length: int,
+            count_steps_by: str = "env_steps",
+            callbacks: "DefaultCallbacks",
+            horizon: int = None,
+            multiple_episodes_in_batch: bool = False,
+            tf_sess=None,
+            clip_actions: bool = True,
+            soft_horizon: bool = False,
+            no_done_at_end: bool = False,
+            observation_fn: "ObservationFunction" = None,
+            _use_trajectory_view_api: bool = False,
+            sample_collector_class: Optional[Type[SampleCollector]] = None):
         """Initializes a SyncSampler object.
 
         Args:
@@ -177,6 +181,9 @@ class SyncSampler(SamplerInput):
             _use_trajectory_view_api (bool): Whether to use the (experimental)
                 `_use_trajectory_view_api` to make generic trajectory views
                 available to Models. Default: False.
+            sample_collector_class (Optional[Type[SampleCollector]]): An
+                optional Samplecollector sub-class to use to collect, store,
+                and retrieve environment-, model-, and sampler data.
         """
 
         self.base_env = BaseEnv.to_base_env(env)
@@ -189,9 +196,15 @@ class SyncSampler(SamplerInput):
         self.extra_batches = queue.Queue()
         self.perf_stats = _PerfStats()
         if _use_trajectory_view_api:
-            self.sample_collector = _SimpleListCollector(
-                policies, clip_rewards, callbacks, multiple_episodes_in_batch,
-                rollout_fragment_length)
+            if not sample_collector_class:
+                sample_collector_class = SimpleListCollector
+            self.sample_collector = sample_collector_class(
+                policies,
+                clip_rewards,
+                callbacks,
+                multiple_episodes_in_batch,
+                rollout_fragment_length,
+                count_steps_by=count_steps_by)
         else:
             self.sample_collector = None
 
@@ -244,26 +257,30 @@ class AsyncSampler(threading.Thread, SamplerInput):
     from where they can be unqueued by the caller of `get_data()`.
     """
 
-    def __init__(self,
-                 *,
-                 worker: "RolloutWorker",
-                 env: BaseEnv,
-                 policies: Dict[PolicyID, Policy],
-                 policy_mapping_fn: Callable[[AgentID], PolicyID],
-                 preprocessors: Dict[PolicyID, Preprocessor],
-                 obs_filters: Dict[PolicyID, Filter],
-                 clip_rewards: bool,
-                 rollout_fragment_length: int,
-                 callbacks: "DefaultCallbacks",
-                 horizon: int = None,
-                 multiple_episodes_in_batch: bool = False,
-                 tf_sess=None,
-                 clip_actions: bool = True,
-                 blackhole_outputs: bool = False,
-                 soft_horizon: bool = False,
-                 no_done_at_end: bool = False,
-                 observation_fn: "ObservationFunction" = None,
-                 _use_trajectory_view_api: bool = False):
+    def __init__(
+            self,
+            *,
+            worker: "RolloutWorker",
+            env: BaseEnv,
+            policies: Dict[PolicyID, Policy],
+            policy_mapping_fn: Callable[[AgentID], PolicyID],
+            preprocessors: Dict[PolicyID, Preprocessor],
+            obs_filters: Dict[PolicyID, Filter],
+            clip_rewards: bool,
+            rollout_fragment_length: int,
+            count_steps_by: str = "env_steps",
+            callbacks: "DefaultCallbacks",
+            horizon: int = None,
+            multiple_episodes_in_batch: bool = False,
+            tf_sess=None,
+            clip_actions: bool = True,
+            blackhole_outputs: bool = False,
+            soft_horizon: bool = False,
+            no_done_at_end: bool = False,
+            observation_fn: "ObservationFunction" = None,
+            _use_trajectory_view_api: bool = False,
+            sample_collector_class: Optional[Type[SampleCollector]] = None,
+    ):
         """Initializes a AsyncSampler object.
 
         Args:
@@ -282,6 +299,8 @@ class AsyncSampler(threading.Thread, SamplerInput):
             rollout_fragment_length (int): The length of a fragment to collect
                 before building a SampleBatch from the data and resetting
                 the SampleBatchBuilder object.
+            count_steps_by (str): Either "env_steps" or "agent_steps".
+                Refers to the unit of `rollout_fragment_length`.
             callbacks (Callbacks): The Callbacks object to use when episode
                 events happen during rollout.
             horizon (Optional[int]): Hard-reset the Env
@@ -305,6 +324,9 @@ class AsyncSampler(threading.Thread, SamplerInput):
             _use_trajectory_view_api (bool): Whether to use the (experimental)
                 `_use_trajectory_view_api` to make generic trajectory views
                 available to Models. Default: False.
+            sample_collector_class (Optional[Type[SampleCollector]]): An
+                optional Samplecollector sub-class to use to collect, store,
+                and retrieve environment-, model-, and sampler data.
         """
         for _, f in obs_filters.items():
             assert getattr(f, "is_concurrent", False), \
@@ -335,9 +357,15 @@ class AsyncSampler(threading.Thread, SamplerInput):
         self.observation_fn = observation_fn
         self._use_trajectory_view_api = _use_trajectory_view_api
         if _use_trajectory_view_api:
-            self.sample_collector = _SimpleListCollector(
-                policies, clip_rewards, callbacks, multiple_episodes_in_batch,
-                rollout_fragment_length)
+            if not sample_collector_class:
+                sample_collector_class = SimpleListCollector
+            self.sample_collector = sample_collector_class(
+                policies,
+                clip_rewards,
+                callbacks,
+                multiple_episodes_in_batch,
+                rollout_fragment_length,
+                count_steps_by=count_steps_by)
         else:
             self.sample_collector = None
 
@@ -429,7 +457,7 @@ def _env_runner(
         no_done_at_end: bool,
         observation_fn: "ObservationFunction",
         _use_trajectory_view_api: bool = False,
-        _sample_collector: Optional[_SampleCollector] = None,
+        sample_collector: Optional[SampleCollector] = None,
 ) -> Iterable[SampleBatchType]:
     """This implements the common experience collection logic.
 
@@ -468,8 +496,8 @@ def _env_runner(
         _use_trajectory_view_api (bool): Whether to use the (experimental)
             `_use_trajectory_view_api` to make generic trajectory views
             available to Models. Default: False.
-        _sample_collector (Optional[_SampleCollector]): An optional
-            _SampleCollector object to use
+        sample_collector (Optional[SampleCollector]): An optional
+            SampleCollector object to use
 
     Yields:
         rollout (SampleBatch): Object containing state, action, reward,
@@ -486,20 +514,27 @@ def _env_runner(
 
     # Trainer has a given `horizon` setting.
     if horizon:
-        # `horizon` is larger than env's limit -> Error and explain how
-        # to increase Env's own episode limit.
+        # `horizon` is larger than env's limit.
         if max_episode_steps and horizon > max_episode_steps:
-            raise ValueError(
-                "Your `horizon` setting ({}) is larger than the Env's own "
-                "timestep limit ({})! Try to increase the Env's limit via "
-                "setting its `spec.max_episode_steps` property.".format(
-                    horizon, max_episode_steps))
+            # Try to override the env's own max-step setting with our horizon.
+            # If this won't work, throw an error.
+            try:
+                base_env.get_unwrapped()[0].spec.max_episode_steps = horizon
+                base_env.get_unwrapped()[0]._max_episode_steps = horizon
+            except Exception:
+                raise ValueError(
+                    "Your `horizon` setting ({}) is larger than the Env's own "
+                    "timestep limit ({}), which seems to be unsettable! Try "
+                    "to increase the Env's built-in limit to be at least as "
+                    "large as your wanted `horizon`.".format(
+                        horizon, max_episode_steps))
     # Otherwise, set Trainer's horizon to env's max-steps.
     elif max_episode_steps:
         horizon = max_episode_steps
         logger.debug(
             "No episode horizon specified, setting it to Env's limit ({}).".
             format(max_episode_steps))
+    # No horizon/max_episode_steps -> Episodes may be infinitely long.
     else:
         horizon = float("inf")
         logger.debug("No episode horizon specified, assuming inf.")
@@ -582,7 +617,7 @@ def _env_runner(
                     soft_horizon=soft_horizon,
                     no_done_at_end=no_done_at_end,
                     observation_fn=observation_fn,
-                    _sample_collector=_sample_collector,
+                    sample_collector=sample_collector,
                 )
         else:
             active_envs, to_eval, outputs = _process_observations(
@@ -616,7 +651,7 @@ def _env_runner(
             eval_results = _do_policy_eval_w_trajectory_view_api(
                 to_eval=to_eval,
                 policies=policies,
-                _sample_collector=_sample_collector,
+                sample_collector=sample_collector,
                 active_episodes=active_episodes,
                 tf_sess=tf_sess,
             )
@@ -641,7 +676,7 @@ def _env_runner(
                 policies=policies,
                 clip_actions=clip_actions,
                 _use_trajectory_view_api=_use_trajectory_view_api,
-                _sample_collector=_sample_collector,
+                sample_collector=sample_collector,
             )
         perf_stats.action_processing_time += time.time() - t3
 
@@ -794,13 +829,13 @@ def _process_observations(
         for agent_id, raw_obs in agent_obs.items():
             assert agent_id != "__all__"
             policy_id: PolicyID = episode.policy_for(agent_id)
-            prep_obs: EnvObsType = _get_or_raise(preprocessors,
-                                                 policy_id).transform(raw_obs)
+            prepr = _get_or_raise(preprocessors, policy_id)
+            prep_obs: EnvObsType = prepr.transform(raw_obs)
             if log_once("prep_obs"):
                 logger.info("Preprocessed obs: {}".format(summarize(prep_obs)))
 
-            filtered_obs: EnvObsType = _get_or_raise(obs_filters,
-                                                     policy_id)(prep_obs)
+            filter = _get_or_raise(obs_filters, policy_id)
+            filtered_obs: EnvObsType = filter(prep_obs)
             if log_once("filtered_obs"):
                 logger.info("Filtered obs: {}".format(summarize(filtered_obs)))
 
@@ -956,7 +991,7 @@ def _process_observations_w_trajectory_view_api(
         soft_horizon: bool,
         no_done_at_end: bool,
         observation_fn: "ObservationFunction",
-        _sample_collector: _SampleCollector,
+        sample_collector: SampleCollector,
 ) -> Tuple[Set[EnvID], Dict[PolicyID, List[PolicyEvalData]], List[Union[
         RolloutMetrics, SampleBatchType]]]:
     """Trajectory View API version of `_process_observations()`.
@@ -975,7 +1010,7 @@ def _process_observations_w_trajectory_view_api(
         episode: MultiAgentEpisode = active_episodes[env_id]
 
         if not is_new_episode:
-            _sample_collector.episode_step(episode.episode_id)
+            sample_collector.episode_step(episode.episode_id)
             episode._add_agent_rewards(rewards[env_id])
 
         # Check episode termination conditions.
@@ -1039,9 +1074,9 @@ def _process_observations_w_trajectory_view_api(
 
             # Record transition info if applicable.
             if last_observation is None:
-                _sample_collector.add_init_obs(episode, agent_id, env_id,
-                                               policy_id, episode.length - 1,
-                                               filtered_obs)
+                sample_collector.add_init_obs(episode, agent_id, env_id,
+                                              policy_id, episode.length - 1,
+                                              filtered_obs)
             else:
                 # Add actions, rewards, next-obs to collectors.
                 values_dict = {
@@ -1067,7 +1102,7 @@ def _process_observations_w_trajectory_view_api(
                 # Env infos for this agent.
                 if "infos" in pol.view_requirements:
                     values_dict["infos"] = agent_infos
-                _sample_collector.add_action_reward_next_obs(
+                sample_collector.add_action_reward_next_obs(
                     episode.episode_id, agent_id, env_id, policy_id,
                     agent_done, values_dict)
 
@@ -1099,7 +1134,7 @@ def _process_observations_w_trajectory_view_api(
             # MultiAgentBatch from a single episode and add it to "outputs".
             # Otherwise, just postprocess and continue collecting across
             # episodes.
-            ma_sample_batch = _sample_collector.postprocess_episode(
+            ma_sample_batch = sample_collector.postprocess_episode(
                 episode,
                 is_done=is_done or (hit_horizon and not soft_horizon),
                 check_dones=check_dones,
@@ -1158,7 +1193,7 @@ def _process_observations_w_trajectory_view_api(
                     new_episode._set_last_observation(agent_id, filtered_obs)
 
                     # Add initial obs to buffer.
-                    _sample_collector.add_init_obs(
+                    sample_collector.add_init_obs(
                         new_episode, agent_id, env_id, policy_id,
                         new_episode.length - 1, filtered_obs)
 
@@ -1171,7 +1206,7 @@ def _process_observations_w_trajectory_view_api(
     # Try to build something.
     if multiple_episodes_in_batch:
         sample_batches = \
-            _sample_collector.try_build_truncated_episode_multi_agent_batch()
+            sample_collector.try_build_truncated_episode_multi_agent_batch()
         if sample_batches:
             outputs.extend(sample_batches)
 
@@ -1267,7 +1302,7 @@ def _do_policy_eval_w_trajectory_view_api(
         *,
         to_eval: Dict[PolicyID, List[PolicyEvalData]],
         policies: Dict[PolicyID, Policy],
-        _sample_collector,
+        sample_collector,
         active_episodes: Dict[str, MultiAgentEpisode],
         tf_sess: Optional["tf.Session"] = None,
 ) -> Dict[PolicyID, Tuple[TensorStructType, StateBatch, dict]]:
@@ -1279,7 +1314,7 @@ def _do_policy_eval_w_trajectory_view_api(
             be the batch's items for the model forward pass).
         policies (Dict[PolicyID, Policy]): Mapping from policy ID to Policy
             obj.
-        _sample_collector (SampleCollector): The SampleCollector object to use.
+        sample_collector (SampleCollector): The SampleCollector object to use.
         tf_sess (Optional[tf.Session]): Optional tensorflow session to use for
             batching TF policy evaluations.
 
@@ -1301,7 +1336,7 @@ def _do_policy_eval_w_trajectory_view_api(
 
     for policy_id, eval_data in to_eval.items():
         policy: Policy = _get_or_raise(policies, policy_id)
-        input_dict = _sample_collector.get_inference_input_dict(policy_id)
+        input_dict = sample_collector.get_inference_input_dict(policy_id)
         eval_results[policy_id] = \
             policy.compute_actions_from_input_dict(
                 input_dict,
@@ -1331,7 +1366,7 @@ def _process_policy_eval_results(
         policies: Dict[PolicyID, Policy],
         clip_actions: bool,
         _use_trajectory_view_api: bool = False,
-        _sample_collector=None,
+        sample_collector=None,
 ) -> Dict[EnvID, Dict[AgentID, EnvActionType]]:
     """Process the output of policy neural network evaluation.
 
