@@ -91,7 +91,9 @@ class Policy(metaclass=ABCMeta):
         if not hasattr(self, "view_requirements"):
             self.view_requirements = view_reqs
         else:
-            self.view_requirements.update(view_reqs)
+            for k, v in view_reqs.items():
+                if k not in self.view_requirements:
+                    self.view_requirements[k] = v
         self._model_init_state_automatically_added = False
 
     @abstractmethod
@@ -289,7 +291,7 @@ class Policy(metaclass=ABCMeta):
             state_batches,
             prev_action_batch=input_dict.get(SampleBatch.PREV_ACTIONS),
             prev_reward_batch=input_dict.get(SampleBatch.PREV_REWARDS),
-            info_batch=None,
+            info_batch=input_dict.get(SampleBatch.INFOS),
             explore=explore,
             timestep=timestep,
             episodes=episodes,
@@ -546,7 +548,8 @@ class Policy(metaclass=ABCMeta):
             model=getattr(self, "model", None),
             num_workers=self.config.get("num_workers", 0),
             worker_index=self.config.get("worker_index", 0),
-            framework=getattr(self, "framework", "tf"))
+            framework=getattr(self, "framework",
+                              self.config.get("framework", "tf")))
         return exploration
 
     def _get_default_view_requirements(self):
@@ -629,10 +632,9 @@ class Policy(metaclass=ABCMeta):
         batch_for_postproc.count = self._dummy_batch.count
         self.exploration.postprocess_trajectory(self, batch_for_postproc)
         postprocessed_batch = self.postprocess_trajectory(batch_for_postproc)
+        seq_lens = None
         if state_outs:
             B = 4  # For RNNs, have B=4, T=[depends on sample_batch_size]
-            # TODO: (sven) This hack will not work for attention net traj.
-            #  view setup.
             i = 0
             while "state_in_{}".format(i) in postprocessed_batch:
                 postprocessed_batch["state_in_{}".format(i)] = \
@@ -642,12 +644,11 @@ class Policy(metaclass=ABCMeta):
                         postprocessed_batch["state_out_{}".format(i)][:B]
                 i += 1
             seq_len = sample_batch_size // B
-            postprocessed_batch["seq_lens"] = \
-                np.array([seq_len for _ in range(B)], dtype=np.int32)
-        # Remove the UsageTrackingDict wrap to prep for wrapping the
-        # train batch with a to-tensor UsageTrackingDict.
-        train_batch = {k: v for k, v in postprocessed_batch.items()}
-        train_batch = self._lazy_tensor_dict(train_batch)
+            seq_lens = np.array([seq_len for _ in range(B)], dtype=np.int32)
+        # Wrap `train_batch` with a to-tensor UsageTrackingDict.
+        train_batch = self._lazy_tensor_dict(postprocessed_batch)
+        if seq_lens is not None:
+            train_batch["seq_lens"] = seq_lens
         train_batch.count = self._dummy_batch.count
         # Call the loss function, if it exists.
         if self._loss is not None:
@@ -671,7 +672,7 @@ class Policy(metaclass=ABCMeta):
                 for key in batch_for_postproc.accessed_keys:
                     if key not in train_batch.accessed_keys and \
                             key in self.view_requirements and \
-                            key not in self.model.inference_view_requirements:
+                            key not in self.model.view_requirements:
                         self.view_requirements[key].used_for_training = False
                 # Remove those not needed at all (leave those that are needed
                 # by Sampler to properly execute sample collection).
@@ -681,7 +682,7 @@ class Policy(metaclass=ABCMeta):
                         SampleBatch.EPS_ID, SampleBatch.AGENT_INDEX,
                         SampleBatch.UNROLL_ID, SampleBatch.DONES,
                         SampleBatch.REWARDS] and \
-                            key not in self.model.inference_view_requirements:
+                            key not in self.model.view_requirements:
                         # If user deleted this key manually in postprocessing
                         # fn, warn about it and do not remove from
                         # view-requirements.
@@ -712,20 +713,40 @@ class Policy(metaclass=ABCMeta):
                 ret[view_col] = \
                     np.zeros((batch_size, ) + shape[1:], np.float32)
             else:
-                if isinstance(view_req.space, gym.spaces.Space):
-                    ret[view_col] = np.zeros_like(
-                        [view_req.space.sample() for _ in range(batch_size)])
+                # Range of indices on time-axis, e.g. "-50:-1".
+                if view_req.shift_from is not None:
+                    ret[view_col] = np.zeros_like([[
+                        view_req.space.sample()
+                        for _ in range(view_req.shift_to -
+                                       view_req.shift_from + 1)
+                    ] for _ in range(batch_size)])
+                # Set of (probably non-consecutive) indices.
+                elif isinstance(view_req.shift, (list, tuple)):
+                    ret[view_col] = np.zeros_like([[
+                        view_req.space.sample()
+                        for t in range(len(view_req.shift))
+                    ] for _ in range(batch_size)])
+                # Single shift int value.
                 else:
-                    ret[view_col] = [view_req.space for _ in range(batch_size)]
+                    if isinstance(view_req.space, gym.spaces.Space):
+                        ret[view_col] = np.zeros_like([
+                            view_req.space.sample() for _ in range(batch_size)
+                        ])
+                    else:
+                        ret[view_col] = [
+                            view_req.space for _ in range(batch_size)
+                        ]
 
-        return SampleBatch(ret)
+        # Due to different view requirements for the different columns,
+        # columns in the resulting batch may not all have the same batch size.
+        return SampleBatch(ret, _dont_check_lens=True)
 
-    def _update_model_inference_view_requirements_from_init_state(self):
+    def _update_model_view_requirements_from_init_state(self):
         """Uses Model's (or this Policy's) init state to add needed ViewReqs.
 
         Can be called from within a Policy to make sure RNNs automatically
         update their internal state-related view requirements.
-        Changes the `self.inference_view_requirements` dict.
+        Changes the `self.view_requirements` dict.
         """
         self._model_init_state_automatically_added = True
         model = getattr(self, "model", None)
@@ -734,11 +755,16 @@ class Policy(metaclass=ABCMeta):
         for i, state in enumerate(obj.get_initial_state()):
             space = Box(-1.0, 1.0, shape=state.shape) if \
                 hasattr(state, "shape") else state
-            view_reqs = model.inference_view_requirements if model else \
+            view_reqs = model.view_requirements if model else \
                 self.view_requirements
             view_reqs["state_in_{}".format(i)] = ViewRequirement(
-                "state_out_{}".format(i), shift=-1, space=space)
-            view_reqs["state_out_{}".format(i)] = ViewRequirement(space=space)
+                "state_out_{}".format(i),
+                shift=-1,
+                batch_repeat_value=self.config.get("model", {}).get(
+                    "max_seq_len", 1),
+                space=space)
+            view_reqs["state_out_{}".format(i)] = ViewRequirement(
+                space=space, used_for_training=True)
 
 
 def clip_action(action, action_space):
